@@ -15,10 +15,12 @@ import stat
 import signal
 from weakref import ref
 
-try:
-    import cpopen
-except ImportError:
-    cpopen = None
+import six
+
+if six.PY2:
+    # subprocess32 improves reliability when using threads.
+    import subprocess32 as subprocess
+else:
     import subprocess
 
 try:
@@ -26,20 +28,11 @@ try:
 except ImportError:
     pthread = None
 
-import six
 Queue = six.moves.queue.Queue
 Empty = six.moves.queue.Empty
 
 elapsed_time = lambda: os.times()[4]  # The system's monotonic timer
 
-try:
-    import vdsm.infra.zombiereaper as zombiereaper
-except ImportError:
-    try:
-        import zombiereaper
-    except ImportError:
-        zombiereaper = None
-
 from . import config
 
 Size = Struct("@Q")
@@ -66,23 +59,9 @@ DEFAULT_MKDIR_MODE = (stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR |
                       stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP |
                       stat.S_IROTH | stat.S_IXOTH)
 
-USE_ZOMBIE_REAPER = False
-
 _ANY_CPU = "0-%d" % (os.sysconf('SC_NPROCESSORS_CONF') - 1)
 
 
-def _spawnProc(cmd):
-    if cpopen:
-        return cpopen.CPopen(cmd)
-    else:
-        return subprocess.Popen(
-            cmd,
-            close_fds=False,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE
-        )
-
-
 # Communicate is a function to prevent the bound method from strong referencing
 # ioproc
 def _communicate(ioproc_ref, proc, readPipe, writePipe):
@@ -192,14 +171,10 @@ def _communicate(ioproc_ref, proc, readPipe, writePipe):
 
         if IOProcess._DEBUG_VALGRIND:
             os.kill(proc.pid, signal.SIGTERM)
-            proc.wait()
         else:
             proc.kill()
 
-        if USE_ZOMBIE_REAPER and zombiereaper is not None:
-            zombiereaper.autoRipPID(proc.pid)
-        else:
-            start_thread(proc.wait, name="ioprocess/%d" % proc.pid)
+        proc.wait()
 
         real_ioproc = ioproc_ref()
         if real_ioproc is not None:
@@ -315,37 +290,40 @@ class IOProcess(object):
     _counter = itertools.count()
 
     def __init__(self, max_threads=0, timeout=60, max_queued_requests=-1,
-                 name=None):
+                 name=None, wait_until_ready=2):
         self.timeout = timeout
         self._max_threads = max_threads
         self._max_queued_requests = max_queued_requests
         self._name = name or "ioprocess-%d" % next(self._counter)
+        self._wait_until_ready = wait_until_ready
         self._commandQueue = Queue()
         self._eventFdReciever, self._eventFdSender = os.pipe()
         self._reqId = 0
         self._isRunning = True
         self._started = Event()
         self._lock = Lock()
+        self._partialLogs = ""
+        self._pid = None
 
         self._log.info("Starting client %s", self.name)
         self._run()
-        self._partialLogs = ""
 
     @property
     def name(self):
         return self._name
 
+    @property
+    def pid(self):
+        return self._pid
+
     def _run(self):
         self._log.debug("Starting ioprocess for client %s", self.name)
         myRead, hisWrite = os.pipe()
         hisRead, myWrite = os.pipe()
 
         for fd in (hisRead, hisWrite):
-            fcntl.fcntl(
-                fd,
-                fcntl.F_SETFD,
-                fcntl.fcntl(fd, fcntl.F_GETFD) & ~(fcntl.FD_CLOEXEC)
-            )
+            # Python 3 creates fds with the close-on-exec flag set.
+            clear_cloexec(fd)
 
         self._partialLogs = ""
 
@@ -366,7 +344,14 @@ class IOProcess(object):
                    "--leak-check=full", "--tool=memcheck"] + cmd + \
                   ["--keep-fds"]
 
-        p = _spawnProc(cmd)
+        p = subprocess.Popen(
+            cmd,
+            pass_fds=(hisRead, hisWrite),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE
+        )
+
+        self._pid = p.pid
 
         os.close(hisRead)
         os.close(hisWrite)
@@ -398,7 +383,7 @@ class IOProcess(object):
             name="ioprocess/%d" % (proc.pid,),
         )
 
-        if self._started.wait(1):
+        if self._started.wait(self._wait_until_ready):
             self._log.debug("Communication thread for client %s started",
                             self.name)
         else:
@@ -508,7 +493,7 @@ class IOProcess(object):
         return self._sendCommand("lexists", {"path": path}, self.timeout)
 
     def fsyncPath(self, path):
-        return self._sendCommand("lexists", {"path": path}, self.timeout)
+        self._sendCommand("fsyncPath", {"path": path}, self.timeout)
 
     def access(self, path, mode):
         try:
@@ -608,6 +593,14 @@ class IOProcess(object):
         self.close(False)
 
 
+def clear_cloexec(fd):
+    """
+    Make fd inheritable by a child process.
+    """
+    flags = fcntl.fcntl(fd, fcntl.F_GETFD)
+    fcntl.fcntl(fd, fcntl.F_SETFD, flags & ~fcntl.FD_CLOEXEC)
+
+
 def start_thread(func, args=(), name=None, daemon=True):
 
     def run():
