@@ -35,6 +35,8 @@ gboolean TRACE_ENABLED = FALSE;
 static int stop_value;
 #define STOP_PTR ((gpointer) &stop_value)
 
+static GAsyncQueue *logger_queue;
+
 static inline void stop_request_reader(void) {
     if (READ_PIPE_FD != -1) {
         close(READ_PIPE_FD);
@@ -127,21 +129,18 @@ static void logfunc(const gchar *log_domain, GLogLevelFlags log_level,
 }
 
 static void *logWriter(void *data) {
-    GAsyncQueue *queue = g_async_queue_new();
     FILE *stream = (FILE *) data;
     size_t nchars = -1;
     size_t n_written = -1;
     char *text = NULL;
 
-    g_log_set_handler(NULL, G_LOG_LEVEL_MASK, logfunc, queue);
-
     while (!feof(stream) && !ferror(stream)) {
         if (n_written == nchars) {
             if (text) {
                 free(text);
                 text = NULL;
             }
-            text = (char*) g_async_queue_pop(queue);
+            text = (char*) g_async_queue_pop(logger_queue);
             n_written = 0;
             nchars = strlen(text);
         } else {
@@ -204,6 +203,7 @@ static int closeUnrelatedFDs(int whitelist[]) {
 
     char fullPath[PATH_MAX];
     char filePath[PATH_MAX];
+    ssize_t len;
 
     /* I use fdopendir so I know what the fd number is so I don't close it mid
      * operation */
@@ -258,9 +258,13 @@ static int closeUnrelatedFDs(int whitelist[]) {
         }
 
         sprintf(fullPath, "/proc/self/fd/%s", fname);
-        if (readlink(fullPath, filePath, PATH_MAX) < 0) {
-            strcpy(fullPath, "(error)");
-        }
+
+        len = readlink(fullPath, filePath, PATH_MAX - 1);
+        if (len < 0)
+            strcpy(filePath, "(error)");
+        else
+            filePath[len] = '\0';
+
         g_trace("Closing unrelated fd no: %s (%s)", fname, filePath);
         if (close(fdNum) < 0) {
             switch (errno) {
@@ -767,6 +771,10 @@ clean:
 GThread *log_writer = NULL;
 
 static int setup_logging() {
+    logger_queue = g_async_queue_new();
+
+    g_log_set_handler(NULL, G_LOG_LEVEL_MASK, logfunc, logger_queue);
+
     log_writer = create_thread("log writer", logWriter, stderr, TRUE);
     if (!log_writer) {
         g_print("Could not allocate request reader thread");
@@ -788,7 +796,8 @@ static void stop_logging() {
 
 int main(int argc, char *argv[]) {
     int rv = 0;
-    int whitelist[] = {1, 2, -1, -1, -1};
+    int whitelist[] = {STDOUT_FILENO, STDERR_FILENO, -1, -1, -1};
+
     if (parseCmdLine(argc, argv) < 0) {
         return -1;
     }
